@@ -151,16 +151,15 @@ def marching_cubes(
     origin = [coords_range[0]] * 3
     spacing = (coords_range[1] - coords_range[0]) / (resolution - 1)
 
-    coords_indices = torch.arange(0, resolution**3, 1)
-    coords = torch.zeros(resolution**3, 4)
+    x = torch.arange(0, resolution)
+    coords_x, coords_y, coords_z = torch.meshgrid(x, x, x, indexing="ij")
+    coords = torch.stack((coords_x, coords_y, coords_z), dim=-1).float()
+    coords *= spacing
+    coords += torch.tensor(origin)
+    coords = coords.reshape(resolution**3, 3)
 
-    coords[:, 2] = coords_indices % resolution
-    coords[:, 1] = (coords_indices.long() / resolution) % resolution
-    coords[:, 0] = ((coords_indices.long() / resolution) / resolution) % resolution
-
-    coords[:, 0] = (coords[:, 0] * spacing) + origin[2]
-    coords[:, 1] = (coords[:, 1] * spacing) + origin[1]
-    coords[:, 2] = (coords[:, 2] * spacing) + origin[0]
+    zeros = torch.zeros(coords.shape[0], 1)
+    coords = torch.cat([coords, zeros], dim=-1)
 
     num_samples = resolution**3
     start = 0
@@ -190,7 +189,8 @@ def marching_cubes(
 def compute_sdf_from_mesh(
     mesh_o3d: o3d.geometry.TriangleMesh,
     method: str = "raycasting",
-    num_surface_points: int = 10_000,
+    num_surface_points: int = 100_000,
+    num_queries_on_surface: int = 10_000,
     queries_stds: List[float] = [0.003, 0.01, 0.1],
     num_queries_per_std: List[int] = [5_000, 4_000, 500, 500],
     coords_range: Tuple[float, float] = (-1.0, 1.0),
@@ -200,10 +200,12 @@ def compute_sdf_from_mesh(
 ) -> Tuple[Tensor, Tensor]:
     """Compute the signed distance function or the occupancy field for a given mesh.
 
-    Query points are sampled by adding gaussian noise to points sampled
-    on the surface of the mesh, according to the given standard deviations.
-    Additionally, query points are also sampled uniformly in the given range.
-    Distances are clipped to max_dist.
+    Query points are sampled in 3 ways:
+    - by sampling point directly from the surface.
+    - by adding gaussian noise to points sampled on the surface of the mesh,
+        according to the given standard deviations.
+    - by sampling points uniformly in the given range.
+    Distances are clipped to (+/-) max_dist.
 
     Two methods are available:
     - raycasting: it works only for watertight meshes.
@@ -213,14 +215,18 @@ def compute_sdf_from_mesh(
     Args:
         mesh_o3d: The open3d mesh.
         method: The method to be used between "raycasting" and "normals".
-        num_surface_points: The number of points to sample from the surface.
+            Defaults to "raycasting".
+        num_surface_points: The number of points to sample from the surface
+            to compute queries. Defaults to 100_000.
+        num_queries_on_surface: The number of queries to sample from the surface.
+            Defaults to 10_000.
         queries_stds: A list of standard deviations to compute the gaussian noise
             to obtain query points. Defaults to [0.003, 0.01, 0.1].
         num_queries_per_std: A list with the number of query points to compute for each
             standard deviation. The last number refers to query points sampled
-            uniformly in the given range (len(num_queries_per_std) = len(queries_std) + 1).
+            uniformly in the given range (i.e., len(num_queries_per_std) = len(queries_std) + 1).
             Defaults to [5_000, 4_000, 500, 500].
-        coords_range: The range for the queries coordinates.
+        coords_range: The range for the queries coordinates. Defaults to (-1 ,1).
         max_dist: The clip value for computed distances. Defaults to 0.1.
         compute_occupancy: If True, compute occupancy instead of sdf. Defaults to False.
         use_cuda: If True, use CUDA to speed up KNN (for method "normals"). Defaults to True.
@@ -240,7 +246,14 @@ def compute_sdf_from_mesh(
         pcd_o3d = mesh_o3d.sample_points_uniformly(number_of_points=num_surface_points)
         pcd = get_tensor_pcd_from_o3d(pcd_o3d)[:, :3]
 
-        queries = sample_points_around_pcd(pcd, queries_stds, num_queries_per_std, coords_range)
+        device = "cuda" if use_cuda else "cpu"
+        queries = sample_points_around_pcd(
+            pcd,
+            queries_stds,
+            num_queries_per_std,
+            coords_range,
+            device,
+        )
 
         if compute_occupancy:
             occ_o3d = scene.compute_occupancy(queries.numpy())
@@ -249,6 +262,17 @@ def compute_sdf_from_mesh(
             sdf_o3d = scene.compute_signed_distance(queries.numpy())
             sdf = torch.tensor(sdf_o3d.numpy())
             values = torch.clip(sdf, min=-max_dist, max=max_dist)
+
+        q_on_surf_o3d = mesh_o3d.sample_points_uniformly(number_of_points=num_queries_on_surface)
+        queries_on_surface = get_tensor_pcd_from_o3d(q_on_surf_o3d)[:, :3]
+
+        if compute_occupancy:
+            values_on_surface = torch.ones(num_queries_on_surface)
+        else:
+            values_on_surface = torch.zeros(num_queries_on_surface)
+
+        queries = torch.cat([queries_on_surface, queries], dim=0)
+        values = torch.cat([values_on_surface, values], dim=0)
     else:
         pcd_o3d = mesh_o3d.sample_points_uniformly(
             number_of_points=num_surface_points,
@@ -257,6 +281,7 @@ def compute_sdf_from_mesh(
 
         queries, values = compute_sdf_from_pcd(
             pcd_o3d,
+            num_queries_on_surface,
             queries_stds,
             num_queries_per_std,
             coords_range,
@@ -266,3 +291,80 @@ def compute_sdf_from_mesh(
         )
 
     return queries, values
+
+
+def compute_udf_from_mesh(
+    mesh_o3d: o3d.geometry.TriangleMesh,
+    num_surface_points: int = 100_000,
+    num_queries_on_surface: int = 10_000,
+    queries_stds: List[float] = [0.003, 0.01, 0.1],
+    num_queries_per_std: List[int] = [5_000, 4_000, 500, 500],
+    coords_range: Tuple[float, float] = (-1.0, 1.0),
+    max_dist: float = 0.1,
+    convert_to_bce_labels: bool = False,
+    use_cuda: bool = True,
+) -> Tuple[Tensor, Tensor]:
+    """Compute the unsigned distance function for a given mesh.
+
+    Query points are sampled in 3 ways:
+    - by sampling point directly from the surface.
+    - by adding gaussian noise to points sampled on the surface of the mesh,
+        according to the given standard deviations.
+    - by sampling points uniformly in the given range.
+    Distances are clipped to max_dist.
+
+    Args:
+        mesh_o3d: The open3d mesh.
+        num_surface_points: The number of points to sample from the surface
+            to compute queries. Defaults to 100_000.
+        num_queries_on_surface: The number of queries to sample from the surface.
+            Defaults to 10_000.
+        queries_stds: A list of standard deviations to compute the gaussian noise
+            to obtain query points. Defaults to [0.003, 0.01, 0.1].
+        num_queries_per_std: A list with the number of query points to compute for each
+            standard deviation. The last number refers to query points sampled
+            uniformly in the given range (i.e., len(num_queries_per_std) = len(queries_std) + 1).
+            Defaults to [5_000, 4_000, 500, 500].
+        coords_range: The range for the queries coordinates. Defaults to (-1, 1).
+        max_dist: The clip value for computed distances. Defaults to 0.1.
+        convert_to_bce_labels: If True, convert distances to labels
+            in the [0, 1] range, with 0=max_dist and 1=surface. Defaults to False.
+        use_cuda: If True, use CUDA to speed up operations. Defaults to True.
+
+    Returns:
+        - The tensor with the query coordinates with shape (N, 3).
+        - The tensor with the computed sdf or occupancy with shape (N,).
+    """
+    scene = o3d.t.geometry.RaycastingScene()
+    vertices = np.asarray(mesh_o3d.vertices, dtype=np.float32)
+    triangles = np.asarray(mesh_o3d.triangles, dtype=np.uint32)
+    _ = scene.add_triangles(vertices, triangles)
+
+    pcd_o3d = mesh_o3d.sample_points_uniformly(number_of_points=num_surface_points)
+    pcd = get_tensor_pcd_from_o3d(pcd_o3d)[:, :3]
+
+    device = "cuda" if use_cuda else "cpu"
+    queries = sample_points_around_pcd(
+        pcd,
+        queries_stds,
+        num_queries_per_std,
+        coords_range,
+        device,
+    )
+
+    udf_o3d = scene.compute_distance(queries.numpy())
+    udf = torch.tensor(udf_o3d.numpy())
+    values = torch.clip(udf, min=0, max=max_dist)
+
+    q_on_surf_o3d = mesh_o3d.sample_points_uniformly(number_of_points=num_queries_on_surface)
+    queries_on_surface = get_tensor_pcd_from_o3d(q_on_surf_o3d)[:, :3]
+    values_on_surface = torch.zeros(num_queries_on_surface)
+
+    queries = torch.cat([queries_on_surface, queries], dim=0)
+    values = torch.cat([values_on_surface, values], dim=0)
+
+    if convert_to_bce_labels:
+        values /= max_dist
+        values = 1 - values
+
+    return queries, values
