@@ -782,7 +782,7 @@ def sample_points_around_pcd(
             to obtain the points.
         num_points_per_std: A list with the number of points to sample for each
             standard deviation. The last number refers to points sampled uniformly
-            in the given range (len(num_points_per_std) = len(stds) + 1).
+            in the given range (i.e., len(num_points_per_std) = len(stds) + 1).
         coords_range: The range for the points coordinates.
         device: The device for the sampled points. Defaults to "cpu".
 
@@ -824,29 +824,34 @@ def sample_points_around_pcd(
 
 def compute_sdf_from_pcd(
     pcd_o3d: o3d.geometry.PointCloud,
+    num_queries_on_surface: int = 10_000,
     queries_stds: List[float] = [0.003, 0.01, 0.1],
     num_queries_per_std: List[int] = [5_000, 4_000, 500, 500],
-    coords_range: Tuple[float, float] = (-0.5, 0.5),
+    coords_range: Tuple[float, float] = (-1, 1),
     max_dist: float = 0.1,
     compute_occupancy: bool = False,
     use_cuda: bool = True,
 ) -> Tuple[Tensor, Tensor]:
     """Compute the signed distance function or the occupancy field for a given pcd.
 
-    The function works only for cloud with accurate normals.
-    Query points are sampled by adding gaussian noise to the cloud, according
-    to the given standard deviations. Additionally, query points are also sampled
-    uniformly in the given range. Distances are clipped to max_dist.
+    The function works only for clouds with accurate normals.
+    Query points are sampled in 3 ways:
+    - by sampling points directly from the surface (i.e., the input cloud).
+    - by adding gaussian noise to the cloud, according to the given standard deviations.
+    - by sampling points uniformly in the given range.
+    Distances are clipped to (+/-) max_dist.
 
     Args:
         pcd_o3d: The open3d point cloud.
+        num_queries_on_surface: Number of points to sample from the input cloud.
+            Defaults to 10_000.
         queries_stds: A list of standard deviations to compute the gaussian noise
             to obtain query points. Defaults to [0.003, 0.01, 0.1].
         num_queries_per_std: A list with the number of query points to compute for each
             standard deviation. The last number refers to query points sampled
-            uniformly in the given range (len(num_queries_per_std) = len(queries_stds) + 1).
+            uniformly in the given range (i.e., len(num_queries_per_std) = len(queries_stds) + 1).
             Defaults to [5_000, 4_000, 500, 500].
-        coords_range: The range for the queries coordinates.
+        coords_range: The range for the queries coordinates. Defaults to (-1, 1).
         max_dist: The clip value for computed distances. Defaults to 0.1.
         compute_occupancy: If True, compute occupancy instead of sdf. Defaults to False.
         use_cuda: If True, use CUDA to speed up KNN. Defaults to True.
@@ -861,9 +866,12 @@ def compute_sdf_from_pcd(
     pcd = pcd_and_normals[:, :3]
     normals = pcd_and_normals[:, 3:]
 
-    k = 11  # from DeepSDF
-
     device = "cuda" if use_cuda else "cpu"
+
+    queries_on_surface = random_point_sampling(pcd.to(device), num_queries_on_surface).cpu()
+    distances_on_surface = torch.zeros(num_queries_on_surface)
+
+    k = 11  # from DeepSDF
     queries = sample_points_around_pcd(pcd, queries_stds, num_queries_per_std, coords_range, device)
     indices, closest_points, distances = knn(queries, pcd.to(device), k)
     queries = queries.cpu()
@@ -881,46 +889,57 @@ def compute_sdf_from_pcd(
     sdf = torch.min(distances[valid_mask], dim=-1)[0]
     sdf[inside] *= -1
 
+    queries = queries[valid_mask]
+
+    if queries.shape[0] != sum(num_queries_per_std):
+        queries_and_sdf = torch.cat((queries, sdf.unsqueeze(-1)), dim=-1)
+        queries_and_sdf = random_point_sampling(
+            queries_and_sdf.to(device),
+            sum(num_queries_per_std),
+        )
+        queries = queries_and_sdf[:, :3]
+        sdf = queries_and_sdf[:, -1].squeeze(-1)
+
+    queries = torch.cat([queries_on_surface, queries], dim=0)
+    distances = torch.cat([distances_on_surface, sdf], dim=0)
+
     if compute_occupancy:
         values = (sdf < 0).float()
     else:
         values = torch.clip(sdf, min=-max_dist, max=max_dist)
 
-    queries = queries[valid_mask]
-
-    if queries.shape[0] != sum(num_queries_per_std):
-        queries_and_values = torch.cat((queries, values.unsqueeze(-1)), dim=-1)
-        queries_and_values = random_point_sampling(queries_and_values, sum(num_queries_per_std))
-        queries = queries_and_values[:, :3]
-        values = queries_and_values[:, -1].squeeze(-1)
-
     return queries, values
 
 
 def compute_udf_from_pcd(
     pcd: Tensor,
+    num_queries_on_surface: int = 10_000,
     queries_stds: List[float] = [0.003, 0.01, 0.1],
     num_queries_per_std: List[int] = [5_000, 4_000, 500, 500],
-    coords_range: Tuple[float, float] = (-0.5, 0.5),
+    coords_range: Tuple[float, float] = (-1, 1),
     max_dist: float = 0.1,
     convert_to_bce_labels: bool = False,
     use_cuda: bool = True,
 ) -> Tuple[Tensor, Tensor]:
     """Compute the unsigned distance function for a given pcd.
 
-    Query points are sampled by adding gaussian noise to the cloud, according
-    to the given standard deviations. Additionally, query points are also sampled
-    uniformly in the given range. Distances are clipped to max_dist.
+    Query points are sampled in 3 ways:
+    - by sampling points directly from the surface (i.e., the input cloud).
+    - by adding gaussian noise to the cloud, according to the given standard deviations.
+    - by sampling points uniformly in the given range.
+    Distances are clipped to max_dist.
 
     Args:
         pcd: The tensor with the point cloud with shape (N, 3).
+        num_queries_on_surface: Number of points to sample from the input cloud.
+            Defaults to 10_000.
         queries_stds: A list of standard deviations to compute the gaussian noise
             to obtain query points. Defaults to [0.003, 0.01, 0.1].
         num_queries_per_std: A list with the number of query points to compute for each
             standard deviation. The last number refers to query points sampled
-            uniformly in the given range (len(num_queries_per_std) = len(queries_stds) + 1).
+            uniformly in the given range (i.e., len(num_queries_per_std) = len(queries_stds) + 1).
             Defaults to [5_000, 4_000, 500, 500].
-        coords_range: The range for the queries coordinates.
+        coords_range: The range for the queries coordinates. Defaults to (-1, 1).
         max_dist: The clip value for computed distances. Defaults to 0.1.
         convert_to_bce_labels: If True, convert distances to labels
             in the [0, 1] range, with 0=max_dist and 1=surface. Defaults to False.
@@ -931,11 +950,18 @@ def compute_udf_from_pcd(
         - The tensor with the computed sdf or occupancy with shape (N,).
     """
     device = "cuda" if use_cuda else "cpu"
+
+    queries_on_surface = random_point_sampling(pcd.to(device), num_queries_on_surface).cpu()
+    distances_on_surface = torch.zeros(num_queries_on_surface)
+
     queries = sample_points_around_pcd(pcd, queries_stds, num_queries_per_std, coords_range, device)
     _, _, distances = knn(queries, pcd.to(device), 1)
     queries = queries.cpu()
     distances = distances.cpu().squeeze(-1)
 
+    queries = torch.cat([queries_on_surface, queries], dim=0)
+    distances = torch.cat([distances_on_surface, distances], dim=0)
+
     values = torch.clip(distances, min=0, max=max_dist)
 
     if convert_to_bce_labels:
@@ -1060,3 +1086,31 @@ def sample_pcds_from_udfs(
         remaining = [i for i in range(batch_size) if pcds[i].shape[0] < num_out_points]
 
     return torch.stack(pcds, dim=0)
+
+
+def backproject(depths: Tensor, K: Tensor) -> Tensor:
+    """Backproject a depthmap to 3D space as organized point clouds.
+
+    Args:
+        depths: The input depth map points with shape ([B,] H, W).
+        K: The intrisics parameter with shape (3, 3).
+
+    Returns:
+        The organized point clouds ([B,] H, W, 3).
+    """
+    batched, [depths] = batchify([depths], 3)
+
+    fx, fy, cx, cy = K[0, 0], K[1, 1], K[0, 2], K[1, 2]
+    w, h = depths.shape[1], depths.shape[2]
+
+    x, y = torch.meshgrid(torch.arange(w), torch.arange(h), indexing="xy")
+    x = x - cx
+    y = y - cy
+    x = repeat(x, "w h -> b w h", b=len(depths))
+    y = repeat(y, "w h -> b w h", b=len(depths))
+
+    pcds = torch.stack((x * depths / fx, y * depths / fy, depths), dim=-1)
+    if batched:
+        [pcds] = unbatchify([pcds])
+
+    return pcds
