@@ -45,12 +45,12 @@ class PCN_encoder(nn.Module):
         x = self.conv2(x)
         global_feature, _ = torch.max(x, 2)
         x = torch.cat(
-            (x, global_feature.view(batch_size, -1, 1).repeat(1, 1, num_points).contiguous()), 1
+            (x, global_feature.reshape(batch_size, -1, 1).repeat(1, 1, num_points).contiguous()), 1
         )
         x = F.relu(self.conv3(x))
         x = self.conv4(x)
         global_feature, _ = torch.max(x, 2)
-        return global_feature.view(batch_size, -1)
+        return global_feature.reshape(batch_size, -1)
 
 
 class PCN_decoder(nn.Module):
@@ -118,7 +118,7 @@ class PCN_decoder(nn.Module):
         grid_y = torch.linspace(-grid_size, grid_size, steps=num_y)
 
         x, y = torch.meshgrid(grid_x, grid_y, indexing="ij")  # x, y shape: (2, 1)
-        grid = torch.stack([x, y], dim=-1).view(-1, 2).transpose(0, 1).contiguous()
+        grid = torch.stack([x, y], dim=-1).reshape(-1, 2).transpose(0, 1).contiguous()
         return grid
 
     def forward(self, x: Tensor) -> Tuple[Tensor, Tensor]:
@@ -135,7 +135,7 @@ class PCN_decoder(nn.Module):
         batch_size = x.size()[0]
         coarse = F.relu(self.fc1(x))
         coarse = F.relu(self.fc2(coarse))
-        coarse = self.fc3(coarse).view(-1, 3, self.num_coarse)
+        coarse = self.fc3(coarse).reshape(-1, 3, self.num_coarse)
 
         grid = self.grid.clone().detach()
         grid_feat = grid.unsqueeze(0).repeat(batch_size, 1, self.num_coarse).contiguous()
@@ -146,7 +146,7 @@ class PCN_decoder(nn.Module):
                 (coarse.transpose(1, 2).contiguous())  # type: ignore
                 .unsqueeze(2)
                 .repeat(1, 1, self.scale, 1)
-                .view(-1, self.num_fine, 3)
+                .reshape(-1, self.num_fine, 3)
             )
             .transpose(1, 2)
             .contiguous()
@@ -161,7 +161,7 @@ class PCN_decoder(nn.Module):
                 (coarse.transpose(1, 2).contiguous())  # type: ignore
                 .unsqueeze(2)
                 .repeat(1, 1, self.scale, 1)
-                .view(-1, self.num_fine, 3)
+                .reshape(-1, self.num_fine, 3)
             )
             .transpose(1, 2)
             .contiguous()
