@@ -13,6 +13,7 @@ def emd(
     eps: float,
     iterations: int,
     squared: bool,
+    reduction: str,
 ) -> Tuple[torch.Tensor, torch.Tensor]:
     """Compute the Earth Mover's Distance.
 
@@ -27,13 +28,14 @@ def emd(
         eps: the balance between the error rate and the speed of convergence.
         iterations: The max number of iterations.
         squared: If true return the squared euclidean distance.
+        reduction: Specifies the reduction to apply to the output: 'none' | 'mean' | 'sum'.
 
     Raises:
         ModuleNotFoundError: If the cuda extension is not installed.
 
     Returns:
         A tuple containing:
-        - A tensor with the average computed distance according to Earth Mover's assignment.
+        - A tensor with the average/sum computed distance according to Earth Mover's assignment.
         - A tensor with the assigned index for each point.
     """
     if find_spec("emdcuda") is None:
@@ -55,7 +57,12 @@ def emd(
     if not squared:
         distances = torch.sqrt(distances)
 
-    distances = distances.mean(1)
+    if reduction == "sum":
+        distances = distances.sum(1)
+
+    if reduction == "mean":
+        distances = distances.mean(1)
+
     if batched:
         distances, assignments = unbatchify([distances, assignments])
 
