@@ -7,7 +7,7 @@ from torch import nn
 
 class Metric(nn.Module, ABC):
     def __init__(self) -> None:
-        super().__init__()  # type: ignore
+        super().__init__()
         self.reset()
 
     def forward(self, predictions: torch.Tensor, targets: torch.Tensor) -> None:
